@@ -490,8 +490,7 @@ ggv_control_button_press_event(GtkWidget *widget, GdkEventButton *event,
 	}
 	return FALSE;
 }
-
-static gboolean
+ gboolean
 ggv_control_popup_menu(GtkWidget *widget, gpointer data)
 {
 	GgvControl *control = GGV_CONTROL(data);
@@ -499,7 +498,7 @@ ggv_control_popup_menu(GtkWidget *widget, gpointer data)
 	return bonobo_control_do_popup(BONOBO_CONTROL(control), 3, GDK_CURRENT_TIME);
 }
 
-static gboolean
+ gboolean
 ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 							gpointer data)
 {
@@ -509,7 +508,6 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
 	GgvPostScriptView *ps_view = control->priv->ps_view;
 	gfloat val;
-
 	/* ugh. the possibilities! */
 	switch (key) {
 	case GDK_space:
@@ -590,14 +588,22 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 		}
 		break;
 	case GDK_Left:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE) &&
-		   ggv_postscript_view_get_page_flip(ps_view))
-			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
+		if((event->state & GDK_MOD1_MASK) == GDK_MOD1_MASK)
+			ggv_postscript_view_goto_page(ps_view,
+										  gtk_gs_get_current_page(gs) - 1);
+		else if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE) &&
+				ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view,
+										  gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Right:
-		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE) &&
-		   ggv_postscript_view_get_page_flip(ps_view))
-			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
+		if((event->state & GDK_MOD1_MASK) == GDK_MOD1_MASK)
+			ggv_postscript_view_goto_page(ps_view,
+										  gtk_gs_get_current_page(gs) + 1);
+		else if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE) &&
+				ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view,
+										  gtk_gs_get_current_page(gs) + 1);		                                                                                                          
 		break;
 	case GDK_Up:
 		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE) &&
@@ -609,6 +615,14 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 		   ggv_postscript_view_get_page_flip(ps_view))
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
+	case GDK_Home:
+	case GDK_KP_Home:
+		ggv_postscript_view_goto_page(ps_view,0);
+		break;
+	case GDK_End:
+	case GDK_KP_End:
+		ggv_postscript_view_goto_page(ps_view,gtk_gs_get_page_count(gs) - 1);
+		break;
 	case GDK_Page_Up:
 	case GDK_KP_Page_Up:
 		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
@@ -617,13 +631,37 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	case GDK_KP_Page_Down:
 		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
+	case GDK_KP_Add:
 	case GDK_plus:
-		g_signal_emit_by_name(G_OBJECT (control->priv->zoomable),
-							  "zoom_in");
+		if((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+			g_signal_emit_by_name(G_OBJECT (control->priv->zoomable),
+								  "zoom_in");
+		else return FALSE;
 		break;
 	case GDK_minus:
-		g_signal_emit_by_name(G_OBJECT(control->priv->zoomable),
-							  "zoom_out");
+	case GDK_KP_Subtract:
+		if((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+			g_signal_emit_by_name(G_OBJECT(control->priv->zoomable),
+								  "zoom_out");
+		else return FALSE;
+		break;
+	case GDK_g:
+		if((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+			g_signal_emit_by_name (G_OBJECT (control->priv->zoomable),
+								   "zoom_to_default");
+		else return FALSE;
+		break;
+	case GDK_f:
+		if((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+			g_signal_emit_by_name (G_OBJECT (control->priv->zoomable),
+								   "zoom_to_fit");
+		else return FALSE;
+		break;
+	case GDK_d:
+		if((event->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+			g_signal_emit_by_name (G_OBJECT (control->priv->zoomable),
+								   "set_zoom_level");
+		else return FALSE;
 		break;
 	default:
 		return FALSE;
